/*
  compilation:
  g++ `pkg-config --libs opencv --cflags opencv` -lglut -lGLU <thisfile>.cpp <classesfile> -o <output file>
*/

/*OpenGL - include headers in this order!*/
#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <GL/freeglut.h>
#include <GL/gl.h>
#include <GL/glu.h>
	
/*OpenCV*/
#include "cv.h"
#include "highgui.h"

/*global variables*/
float widthscale = 0.05, heightscale = 0.05;	//how many OpenGL coordinate units per pixel?
int win_width = 1000, win_height = 1000;			//initial window size in pixels
float move_rate = 10;							//hero move rate target in OpenGL coordinates per second
float move_amnt = .1;							//current hero move amount in OpenGL coordinates per frame
float move_amnt_lim = .2;					//never move more than this many OpenGL coords, prevents clipping
int currentscene = 1;	//initialize to scene 1
int newscene = 1;	//default assumption is that it's the first time we're entering a scene
int next_scene = 1; //scene to transition to - initialize to first scene (1)
int movement[4] = {0,0,0,0}; //WASD: up, left, down, right
const unsigned char teststr[10] = {'T','E','S','T','_','S','T','R','*',0};
struct timeval time0; //get timing info, use to adjust movement
struct timeval time1;
double timediff;

/*Custom Game Classes*/
#include "vgEngineClasses.h"
#include "scenes.cpp"

/*we'll make these global, too*/
sprite hero;	//sprite
background bg;	//background

/*prototypes*/
int moveplayer(void);
int initnewscene(int next_scene);

void init(void)
{
  glClearColor(0.2,0.4,0.8,0); //clear color 
  glShadeModel(GL_FLAT);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_TEXTURE_2D);

  return;
}



void display(void)
{

  //printf("\ncurrentscene = %d\n",currentscene);

  switch (currentscene){
  case 1:{	//scene1
    if (newscene){
      initnewscene(next_scene);
    }
    scene1();
    break;
  }
  case 2:{	//scene2
    if (newscene){
      initnewscene(next_scene);
      newscene=0;
    }
    scene2();
    break;
  }
  default:{
    break;
  }		
  }

	/*get timing info to adjust movement*/
	gettimeofday(&time1, NULL);
	//time diff in sec
	timediff = time1.tv_sec + time1.tv_usec*0.000001 - time0.tv_sec - time0.tv_usec*0.000001;
	move_amnt = timediff*move_rate; //should move a distance proportional to time step
																	//assuming constant velocity
	//printf("timediff: %lf\n",timediff); //NOTE: PRINT STATEMENTS CAUSE JITTERY PERFORMANCE
	if(move_amnt > move_amnt_lim){
		move_amnt = move_amnt_lim;
	}
	if(move_amnt < 0){
		move_amnt = 0;
	}
  moveplayer(); //update player location
	gettimeofday(&time0, NULL); //update time0 for next time
	
	glutPostRedisplay(); //force redraw
  glutSwapBuffers();

}


int moveplayer(){
     
  if(movement[0]){
    hero.trySetXYZ(hero.x,hero.y+move_amnt,hero.z,&bg);
    if(hero.facing){
      hero.setFacing(0);
      hero.setTheta(0);
    }
	 
  }
  if(movement[1]){
    hero.trySetXYZ(hero.x-move_amnt,hero.y,hero.z,&bg);
    if(hero.facing != 3){
      hero.setFacing(3);
      hero.setTheta(90);
    }
			
  }
  if(movement[2]){
    hero.trySetXYZ(hero.x,hero.y-move_amnt,hero.z,&bg);
    if(hero.facing != 2){
      hero.setFacing(2);
      hero.setTheta(180);
    }
		   
  }
  if(movement[3]){
    hero.trySetXYZ(hero.x+move_amnt,hero.y,hero.z,&bg);
    if(hero.facing != 1){
      hero.setFacing(1);
      hero.setTheta(-90);
    }
		
  }
	 
  return 0;

}

int initnewscene(int next_scene){
	switch(next_scene){
	case 1:{
		scene1init();
		break;
	}
	case 2:{
		scene2init();
		break;
	}
	default:{
		break;
	}
	}

	newscene = 0;
	return 0;
}


void keyboard(unsigned char key, int x, int y){

  switch(key){
  case 'w':{
    movement[0] = 1;
    break;
  }
  case 'a':{
    movement[1] = 1;
    break;
  }
  case 's':{
    movement[2]=1;
    break;
  }
  case 'd':{
    movement[3]=1;
    break;
  }
  default:{
    break;
  }
  }
}



void keyboardup(unsigned char key, int x, int y){

  switch(key){
  case 'w':{
    movement[0] = 0;
    break;
  }
  case 'a':{
    movement[1] = 0;
    break;
  }
  case 's':{
    movement[2]=0;
    break;
  }
  case 'd':{
    movement[3]=0;
    break;
  }
  default:{
    break;
  }
  }
}



void reshape(int w, int h)
{
  glViewport(0, 0, (GLsizei) w, (GLsizei) h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(-w*widthscale*0.5,w*widthscale*0.5,-h*widthscale*0.5,h*widthscale*0.5,0.0,60.0);
  glMatrixMode(GL_MODELVIEW);
}



int main(int argc, char** argv)
{
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
  glutInitWindowSize (win_width, win_height);
  glutInitWindowPosition (100, 100);
  glutCreateWindow (argv[0]);
  init();
  glutDisplayFunc(display);
  glutReshapeFunc(reshape);
  glutKeyboardFunc(keyboard);
  glutKeyboardUpFunc(keyboardup);

	gettimeofday(&time0, NULL); //initial time0 value

  glutMainLoop();

  return 0;
}

